### Licence: LGPL version 2.1
## Time-stamp: "Last modified 2018-08-29 15:15:50 delucia"

db <- RPhreeFile(system.file("extdata", "phreeqc_kin.dat", package="RedModRphree"), is.db=TRUE)
phreeqc::phrLoadDatabaseString(db)

selec <- c(
    "SELECTED_OUTPUT",
    "-high_precision true",
    "-reset false", 
    "USER_PUNCH",
    "-head C Ca Cl Mg pH pe Calcite Dolomite",
    "10 PUNCH TOT(\"C(4)\"), TOT(\"Ca\"), TOT(\"Cl\"), TOT(\"Mg\"), -LA(\"H+\"), -LA(\"e-\"), EQUI(\"Calcite\"), EQUI(\"Dolomite\")")

base <- c(
    "SOLUTION 1",
    "units mol/kgw",
    "temp 25.0",
    "pH 9.91 charge",
    "pe 4.0  O2(g) -0.68",
    "C(4)   1.2279E-04",
    "Ca   1.2279E-04",
    "Mg 1.0E-12",
    "Cl 1.0E-12",
    "PURE 1",
    "Calcite  0.0  2.07E-04",
    "Dolomite 0.0  0.00E+00",
    "END")

maxsim <- 50         ## minimum number of single chemical simulations in each time step to spawn parallelization
grid   <- 100         ## grid discretization (number of elements)
L      <- 0.5        ## total system length [m]
U      <- 9.375e-6   ## imposed (constant) Darcy velocity [m3/s]
Cf     <- 0.8        ## Courant number to add some numerical dispersion

## boundary conditions (concentration of the injected fluid entering into the 1D column)
bound <- c(H=1E-7, e=1E-4, Mg=0.001, Cl=0.002)
prop <- c("C(4)","Ca","Cl","Mg","pH","pe","Calcite","Dolomite")

## simulations with grid 50, 100 and 200
setup <- list(n=50, len=L, U=U, base=base, first=selec, Cf=Cf, bound=bound, prop=prop,
              immobile=c(7,8), initsim=c(selec, base))

sim50 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
                              procs=4, maxsim=maxsim, writeout=FALSE, 
                              reduce=FALSE, surrogate=FALSE)

setup <- list(n=100, len=L, U=U, base=base, first=selec, Cf=Cf, bound=bound, prop=prop,
              immobile=c(7,8), initsim=c(selec, base))

sim100 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
                              procs=4, maxsim=maxsim, writeout=FALSE, 
                              reduce=FALSE, surrogate=FALSE)

setup <- list(n=200, len=L, U=U, base=base, first=selec, Cf=Cf, bound=bound, prop=prop,
              immobile=c(7,8), initsim=c(selec, base))

sim200 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
                              procs=4, maxsim=maxsim, writeout=FALSE, 
                              reduce=FALSE, surrogate=FALSE)

sam <- ExtractSamples(list(sim50, sim100, sim200),
                      var_design=c("C","Ca","Cl","Mg", "pH", "pe", "Calcite", "Dolomite"),
                      var_result=c("C","Ca","Cl","Mg", "pH", "pe", "Calcite", "Dolomite"))

## This code was used to generate the included csv data
## data.table::fwrite(sam$design, file = "RedModRphree/inst/extdata/demo_equilibrium_design.csv",
##                    append = FALSE, quote = "auto", sep = ",",  eol = "\n", na = "NA", dec = ".",
##                    row.names = FALSE, col.names = TRUE)

## data.table::fwrite(sam$result, file = "RedModRphree/inst/extdata/demo_equilibrium_result.csv",
##                    append = FALSE, quote = "auto",  sep = ",",  eol = "\n", na = "NA", dec = ".",
##                    row.names = FALSE, col.names = TRUE)

## this is how you can import those data without running the whole thing
## sam <- list(
##     design = data.matrix(data.table::fread(file = system.file("extdata", "demo_equilibrium_design.csv", package="RedModRphree"))),
##     result =  data.matrix(data.table::fread(file = system.file("extdata", "demo_equilibrium_result.csv", package="RedModRphree")))
## )


baleq <- list(Calcite=c(C=1, Ca=1), Dolomite=c(C=2, Ca=1, Mg=1))

balance <- CheckBalance(baleq, sam$design, sam$result)

## we import the caret library
library(caret)

## we register the parallelization backend
doParallel::registerDoParallel(4)

set.seed(95014)
fitControl <- trainControl(method = "none",
                           allowParallel = TRUE, 
                           returnData = FALSE,
                           returnResamp="none",
                           savePredictions="none",
                           predictionBounds=rep(TRUE,2))


## modk <- parallel::mclapply(colnames(sam$result), function (x) 
##     return(train(x=sam$design, y=sam$result[,x],trControl = fitControl,  ##tuneGrid = tg, 
##                  method="parRF")), mc.cores=8)

traintime <- system.time({
    mods <- foreach(i=colnames(sam$result)) %dopar% caret::train(x=sam$design, y=sam$result[,i], trControl = fitControl, method="parRF")
})

names(mods) <- colnames(sam$result)

## ## this takes around 8 minutes using 4 cores
## traintime <- system.time({
##     mods <- parallel::mclapply(colnames(sam$result), function (x) 
##         return(train(x=sam$design, y=sam$result[,x],trControl = fitControl,
##                      method="parRF")), mc.cores=7)
##     names(mods) <- colnames(sam$result)
## })

PlotModsInSample(mods, sam$result, sam$design)
## saveRDS(mods, "models_equilibrium_RF.rds")

## fitControl2 <- trainControl(method = "none", 
##                             allowParallel = TRUE,
##                             returnData = FALSE,
##                             returnResamp="none",
##                             savePredictions="none",
##                             verboseIter=FALSE,
##                             predictionBounds=rep(TRUE,2))

## mods2 <- parallel::mclapply(colnames(sam$result), function (x) 
##     return(train(x=sam$design, y=sam$result[,x],trControl = fitControl2,
##                  method="parRF")), mc.cores=8)
## names(mods2) <- colnames(sam$result)
## PlotModsInSample(mods2, sam$result, sam$design)


## to use the surrogate in the reactive transport simulations we need a function
MySurr <- function(state, model){
    order_for_surrogate <- c("C","Ca","Cl","Mg", "pH","pe", "Calcite","Dolomite")
    remember_names <- colnames(state)
    rem_attr <- attr(state,"immobile")
    ss <- state[, order_for_surrogate]
    
    pred <- lapply(names(model), function(x) as.numeric(predict(model[[x]], ss)))
    names(pred) <- names(model)
    out <- data.matrix(as.data.frame(pred))
    ## order back to how it is expected in "state"
    out <- out[,remember_names]
    attr(out,"immobile") <- rem_attr 
    return(out)
}


## now run the simulations with surrogates
setup <- list(n=50, len=L, U=U, base=base, first=selec, Cf=Cf, bound=bound, prop=prop,
              immobile=c(7,8), initsim=c(selec, base))

sur50_4 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
                                procs=4, maxsim=maxsim, writeout=FALSE, 
                                reduce=FALSE, surrogate=TRUE, surrogate.FUN=MySurr,
                                model=mods, baleq=baleq, tol=1E-4, call_pqc=TRUE)

sur50_5 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
                                procs=4, maxsim=maxsim, writeout=FALSE, 
                                reduce=FALSE, surrogate=TRUE, surrogate.FUN=MySurr,
                                model=mods, baleq=baleq, tol=1E-5, call_pqc=TRUE)


## some visualization
par(mfrow=c(2,1))
Matplot(sur=sur50_4, sim=sim50, xlab="Domain element",
        ylab="Ca, Mg [molal], Calcite, Dolomite [mol]",
        main="RandomForest surrogate, tol=1E-4")
Matplot(sur=sur50_5, sim=sim50, xlab="Domain element",
        ylab="Ca, Mg [molal], Calcite, Dolomite [mol]",
        main="RandomForest surrogate, tol=1E-5")

## setup <- list(n=100, len=L, U=U, base=base, first=selec, Cf=Cf, bound=bound, prop=prop,
##               immobile=c(7,8), initsim=c(selec, base))

## sur100_4 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
##                                  procs=4, maxsim=maxsim, writeout=FALSE, 
##                                  reduce=FALSE, surrogate=TRUE, surrogate.FUN=MySurr,
##                                  model=mods, baleq=baleq, tol=1E-4, call_pqc=TRUE)

## sur100_5 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
##                                 procs=4, maxsim=maxsim, writeout=FALSE, 
##                                 reduce=FALSE, surrogate=TRUE, surrogate.FUN=MySurr,
##                                 model=mods, baleq=baleq, tol=1E-5, call_pqc=TRUE)

## setup <- list(n=200, len=L, U=U, base=base, first=selec, Cf=Cf, bound=bound, prop=prop,
##               immobile=c(7,8), initsim=c(selec, base))

## sur200_4 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
##                                 procs=4, maxsim=maxsim, writeout=FALSE, 
##                                 reduce=FALSE, surrogate=TRUE, surrogate.FUN=MySurr,
##                                 model=mods, baleq=baleq, tol=1E-4, call_pqc=TRUE)

## sur200_5 <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
##                                 procs=4, maxsim=maxsim, writeout=FALSE, 
##                                 reduce=FALSE, surrogate=TRUE, surrogate.FUN=MySurr,
##                                 model=mods, baleq=baleq, tol=1E-5, call_pqc=TRUE)

 

## par(mfrow=c(2,1))
## Matplot(50, sur=sur100_4, sim=sim100, xlab="Domain element", ylab="Ca, Mg [molal], Calcite, Dolomite [mol]" ,main="RandomForest surrogate, tol=1E-4")
## Matplot(50, sur=sur100_5, sim=sim100, xlab="Domain element", ylab="Ca, Mg [molal], Calcite, Dolomite [mol]" ,main="RandomForest surrogate, tol=1E-5")

## par(mfrow=c(2,1))
## Matplot(99, sur=sur200_4, sim=sim200, xlab="Domain element", ylab="Ca, Mg [molal], Calcite, Dolomite [mol]" ,main="RandomForest surrogate, tol=1E-4")
## Matplot(99, sur=sur200_5, sim=sim200, xlab="Domain element", ylab="Ca, Mg [molal], Calcite, Dolomite [mol]" ,main="RandomForest surrogate, tol=1E-5")

