### Licence: LGPL version 2.1
### Time-stamp: "Last modified 2018-05-06 19:26:31 delucia"

db <- RPhreeFile(system.file("extdata", "phreeqc_kin.dat",
                             package="RedModRphree"), is.db=TRUE)

phreeqc::phrLoadDatabaseString(db)

 
selec <- c("SELECTED_OUTPUT", "-high_precision true", "-reset false",
           "USER_PUNCH", "-head C Ca Cl Mg pH pe Calcite Dolomite",
           "10 PUNCH TOT(\"C(4)\"), TOT(\"Ca\"), TOT(\"Cl\"), TOT(\"Mg\"), -LA(\"H+\"), -LA(\"e-\"), EQUI(\"Calcite\"), EQUI(\"Dolomite\")")

base <- c("SOLUTION 1", "units mol/kgw", "temp 25.0",
          "pH 9.91 charge", "pe 4.0  O2(g) -0.68",
          "C(4)   1.2279E-04", "Ca   1.2279E-04", "Mg 1.0E-12",
          "Cl 1.0E-12", "PURE 1", "Calcite  0.0  2.07E-04",
          "Dolomite 0.0  0.00E+00", "END")

maxsim <- 50         ## minimum number of single chemical simulations in each time step to spawn parallelization
L      <- 0.5        ## total system length [m]
U      <- 9.375e-6   ## imposed (constant) Darcy velocity [m3/s]
Cf     <- 0.8        ## Courant number to add some numerical dispersion

## boundary conditions (concentration of the injected fluid entering
## into the 1D column)
bound <- c(H=1E-7, e=1E-4, Mg=0.001, Cl=0.002)
prop <- c("C(4)","Ca","Cl","Mg","pH","pe","Calcite","Dolomite")

grid   <- 50         ## grid discretization (number of elements)
setup <- list(n=grid, len=L, U=U, base=base, first=selec, Cf=Cf,
              bound=bound, prop=prop, immobile=c(7,8),
              initsim=c(selec, base))

sime <- ReactTranspBalanceEq(setup=setup, db=db, step="time", maxtime=21000, 
                             procs=4, maxsim=maxsim, writeout=FALSE, 
                             reduce=FALSE, surrogate=FALSE)

MatplotSingle(sim=sime, xlab="Domain element", ylab="Ca, Mg [molal], Calcite, Dolomite [mol]",
              main="Equilibrium simulation, grid 50")

