### RedModRphree, demo of kinetics 

### Licence: LGPL version 2.1
### Time-stamp: "Last modified 2018-05-06 21:26:21 delucia"
library(RedModRphree)
db <- RPhreeFile(system.file("extdata", "phreeqc_kin.dat", package="RedModRphree"), is.db=TRUE)

base <- c("SOLUTION 1", "units mol/kgw", "temp 25.0", "water 1",
          "pH 9.91 charge", "pe 4.0", "C   1.2279E-04",
          "Ca  1.2279E-04", "Mg 0.001", "Cl 0.002", "PURE 1",
          "O2g -0.1675 10", "KINETICS 1", "-steps 100",
          "-bad_step_max 1000", "Calcite", "-m      0.000207",
          "-parms  3.20", "Dolomite", "-m      0.0", "-parms  0.32",
          "END")


initsim <- c("SELECTED_OUTPUT", "-high_precision true",
             "-reset false", "USER_PUNCH",
             "-head C Ca Cl Mg H e O2g Calcite Dolomite",
             "10 PUNCH TOT(\"C\"), TOT(\"Ca\"), TOT(\"Cl\"), TOT(\"Mg\"), ACT(\"H+\"), ACT(\"e-\"), EQUI(\"O2g\"), EQUI(\"Calcite\"), EQUI(\"Dolomite\")",
             "SOLUTION 1", "units mol/kgw", "temp 25.0", "water 1",
             "pH 9.91 charge", "pe 4.0", "C   1.2279E-04",
             "Ca  1.2279E-04", "Cl 1E-12", "Mg 1E-12", "PURE 1",
             "O2g     -0.6788 10.0", "Calcite  0.0 2.07E-3",
             "Dolomite 0.0 0.0", "END")
 

selout <- c("SELECTED_OUTPUT", "-high_precision true", "-reset false",
            "-time", "-soln", "-temperature true", "-water true",
            "-pH", "-pe", "-totals C Ca Cl Mg",
            "-kinetic_reactants Calcite Dolomite", "-equilibrium O2g")

prop <- c("C","Ca","Cl","Mg","pH","pe","O2g", "Calcite","Dolomite")
maxsim <- 20         ## minimum number of single chemical simulations in each time step to spawn parallelization
grid   <- 50         ## grid discretization (number of elements)
L      <- 0.5        ## total system length [m]
U      <- 9.375e-6   ## imposed (constant) Darcy velocity [m3/s]
Cf     <- 0.9        ## Courant number to add some numerical dispersion

## boundary conditions (concentration of the injected fluid entering into the 1D column)
bound <- c(H=1E-7, e=1E-4, Mg=0.001, Cl=0.002)

## pack all together in a "setup" list
setup <- list(n=50, len=L, U=U, base=base, first=selout, Cf=Cf, bound=bound, dt=210,
              prop=prop, immobile=c(7,8,9), kin=c(8,9), ann=list(O2g=-0.1675), initsim=initsim)

simk <- ReactTranspBalanceKin(setup=setup, db=db, step="fix_dt", maxtime=21000,
                              procs=4, maxsim=maxsim, writeout=FALSE, 
                              reduce=FALSE, surrogate=FALSE)


## visualize
MatplotSingle(sim=simk, xlab="Domain element", ylab="Ca, Mg [molal], Calcite, Dolomite [mol]", main="1D simulation with grid 50")
